

local vmajor, vminor = "1", tonumber(string.sub("$Revision: 1776 $", 12, -3))
local stubvarname = "TekLibStub"
local libvarname = "TekTechEmbed"


-- Check to see if an update is needed
-- if not then just return out now before we do anything
local libobj = getglobal(libvarname)
if libobj and not libobj:NeedsUpgraded(vmajor, vminor) then return end


local stubobj = getglobal(stubvarname)
if not stubobj then
	stubobj = {}
	setglobal(stubvarname, stubobj)


	-- Instance replacement method, replace contents of old with that of new
	function stubobj:ReplaceInstance(old, new)
		 for k,v in pairs(old) do old[k]=nil end
		 for k,v in pairs(new) do old[k]=v end
	end


	-- Get a new copy of the stub
	function stubobj:NewStub(name)
		local newStub = {}
		self:ReplaceInstance(newStub, self)
		newStub.libName = name
		newStub.lastVersion = ''
		newStub.versions = {}
		return newStub
	end


	-- Get instance version
	function stubobj:NeedsUpgraded(vmajor, vminor)
		local versionData = self.versions[vmajor]
		if not versionData or versionData.minor < vminor then return true end
	end


	-- Get instance version
	function stubobj:GetInstance(version)
		if not version then version = self.lastVersion end
		local versionData = self.versions[version]
		if not versionData then print(string.format("<%s> Cannot find library version: %s", self.libName, version or "")) return end
		return versionData.instance
	end


	-- Register new instance
	function stubobj:Register(newInstance)
		 local version,minor = newInstance:GetLibraryVersion()
		 self.lastVersion = version
		 local versionData = self.versions[version]
		 if not versionData then
				-- This one is new!
				versionData = {
					instance = newInstance,
					minor = minor,
					old = {},
				}
				self.versions[version] = versionData
				newInstance:LibActivate(self)
				return newInstance
		 end
		 -- This is an update
		 local oldInstance = versionData.instance
		 local oldList = versionData.old
		 versionData.instance = newInstance
		 versionData.minor = minor
		 local skipCopy = newInstance:LibActivate(self, oldInstance, oldList)
		 table.insert(oldList, oldInstance)
		 if not skipCopy then
				for i, old in ipairs(oldList) do self:ReplaceInstance(old, newInstance) end
		 end
		 return newInstance
	end
end


-------------------------------
--      Library Methods      --
-------------------------------

if not libobj then
	libobj = stubobj:NewStub(libvarname)
	setglobal(libvarname, libobj)
end

local lib = {}


-- Return the library's current version
function lib:GetLibraryVersion()
	return vmajor, vminor
end


-- Activate a new instance of this library
function lib:LibActivate(stub, oldLib, oldList)
--	if oldLib then
--	else
--	end
	-- nil return makes stub do object copy
end


function lib:TableGetVal(t, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
	assert(type(t) == "table", "No table passed")
	assert(a1, "No index passed")
	if not t[a1] then return end

	if not a2 then return t[a1] end
	if type(t[a1]) == "table" then return self:TableGetVal(t[a1], a2,a3,a4,a5,a6,a7,a8,a9,a10) end
end


function lib:TableSetVal(t, val, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
	assert(type(t) == "table", "No table passed")
	assert(a1, "No index passed")

	if not a2 then
		t[a1] = val
		local notempty
		for i in pairs(t) do notempty = true end
		if not notempty then t = nil end
	else
		if not t[a1] then t[a1] = {} end
		self:TableSetVal(t[a1], val, a2,a3,a4,a5,a6,a7,a8,a9,a10)
	end
end


function lib:ColorizeCash(money, includezero)
	assert(type(money) == "number", "Money must be a number")
	if not includezero and money == 0 then return end

	local g = math.floor(money / 10000) or 0
	local s = math.floor((money - (g*10000)) / 100) or 0
	local c = math.floor(money - (g*10000) - (s*100)) or 0

	local gt = (g > 0) and string.format("|cffffd000%d|r", g) or ""
	local st = (g > 0) and string.format("|cffe6e6e6.%02d|r", s) or (s > 0) and string.format("|cffe6e6e6%d|r", s) or ""
	local ct = (g > 0 or s > 0) and string.format("|cffc8602c.%02d|r", c) or string.format("|cffc8602c%d|r", c)

	return string.format("%s%s%s", gt, st, ct)
end


function lib:SpellKnown(spell)
	assert(spell, "No spell passed")
	local id, searchName = 1

	repeat
		searchName = GetSpellName(id, BOOKTYPE_SPELL)
		if searchName == spell then return id end
		id = id + 1
	until	not searchName
end


function lib:SpellRankKnown(spell, rank)
	assert(spell, "No spell passed")
	local t = type(rank)
	assert(t == "string" or t == "number", "Invalid rank format passed")

	if t == "number" then rank = string.format("Rank %s", rank) end

	local id, searchName, r = 1
	repeat
		searchName, r = GetSpellName(id, BOOKTYPE_SPELL)
		if searchName == spell and rank == r then return id end
		id = id + 1
	until	not searchName
end



function lib:SpellUsable(spell)
	assert(spell, "No spell passed")

	local id, n = 1
	repeat
		n = GetSpellName(id,BOOKTYPE_SPELL)
		if n == spell then
			local _, cooling = GetSpellCooldown(id, BOOKTYPE_SPELL)
			if cooling == 0 then return true
			else return end
		end
		id = id + 1
	until	not n
end


local bindtypes = {ITEM_BIND_ON_EQUIP, ITEM_BIND_ON_PICKUP, ITEM_BIND_ON_USE, ITEM_BIND_QUEST, ITEM_SOULBOUND}
local gratuity = Gratuity and Gratuity:GetInstance("1")
-- Assumes Gratuity has been set for the item you want to test
function lib:GetBindType()
	assert(Gratuity, "GratuityLib not found")
	gratuity = gratuity or Gratuity:GetInstance("1")

	for _,b in pairs(bindtypes) do
		if gratuity:Find(b, 2, 5) then return b end
	end
end


--------------------------------
--      Load this bitch!      --
--------------------------------
libobj:Register(lib)
